package fpmon.stats;

import fpmon.chart.AAChart2D;
import fpmon.chart.AxisTitlePainterPositioned;
import fpmon.chart.LabelFormatterNull;
import fpmon.chart.XAxisNoOverhang;
import info.monitorenter.gui.chart.*;
import info.monitorenter.gui.chart.labelformatters.LabelFormatterDate;
import info.monitorenter.gui.chart.labelformatters.LabelFormatterNumber;
import info.monitorenter.gui.chart.rangepolicies.RangePolicyFixedViewport;
import info.monitorenter.gui.chart.traces.Trace2DLtd;
import info.monitorenter.gui.chart.traces.painters.TracePainterFill;
import info.monitorenter.util.Range;
import java.awt.Color;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;

/**
 * Displays stats using charts.
 * 
 * @author Miles Asvachin
 */
abstract public class ChartStatCollector extends StatCollector
{
	/**
	 * Format used by x-axes that display the time.
	 */
	protected final SimpleDateFormat timeAxisFormat = new SimpleDateFormat("HH:mm");
	
	/** Number of data points shown on the chart. */
	protected int traceSize = 600;
	/** Title of the main trace. */
	protected String title = null;
	/** Colour of the main trace. */
	protected Color traceColor = Color.WHITE;
	/** Transparency of the fill trace added under line traces. */
	protected int fillAlpha = 64;
	/** Space to pad the right hand side with, so the chart's x-axis doesn't touch the edge. */
	protected int rightBorder = 4;
	/**
	 * Charts with multiple traces should pad all but the last title to this 
	 * length, to minimize how much they would otherwise move around when their 
	 * length changes. Simple padding doesn't work perfectly since the font is 
	 * proportional, but it's good enough.
	 */
	protected int traceTitleLength = 20;

	/**
	 * Helper function for creating a basic line chart.
	 */
	protected static Chart2D createLineChart(
			final String newTitle,
			final Color newBackgroundColor,
			final Color newForegroundColor)
	{
		return createChart(newTitle, newBackgroundColor, newForegroundColor, true);
	}
	
	/**
	 * Helper function for creating a chart intended for showing a single bar.
	 */
	protected static Chart2D createBarChart(
			final String newTitle,
			final Color newBackgroundColor,
			final Color newForegroundColor)
	{
		return createChart(newTitle, newBackgroundColor, newForegroundColor, false);
	}
	
	/**
	 * Creates a chart and sets it up.
	 */
	private static Chart2D createChart(
			final String newTitle,
			final Color newBackgroundColor,
			final Color newForegroundColor,
			final boolean useAntialiasing)
	{
		final Chart2D chart = new AAChart2D();
		chart.setBackground(newBackgroundColor);
		chart.setForeground(newForegroundColor);
		chart.setUseAntialiasing(useAntialiasing);
		chart.setCursor(null);
		
		chart.setAxisXBottom(new XAxisNoOverhang(), 0);
		
		final IAxis xAxis = chart.getAxisX();
		xAxis.getAxisTitle().setTitleColor(newForegroundColor.darker().darker());

		final IAxis yAxis = chart.getAxisY();
		yAxis.getAxisTitle().setTitleColor(newForegroundColor);
		
		chart.getAxisX().getAxisTitle().setTitle(newTitle);
		
		if(newTitle != null)
		{
			chart.getAxisX().getAxisTitle().setTitlePainter(new AxisTitlePainterPositioned(0.5, 1.0, 0.5, 0.0));
		}
		
		return chart;
	}
	
	/**
	 * Configures the axis to show time
	 */
	protected static void axisTime(final IAxis axis, final SimpleDateFormat format)
	{
		axis.setFormatter(new LabelFormatterDate(format));
		axis.setMinorTickSpacing(6.0);
		axis.setMajorTickSpacing(30.0);
	}
	
	/**
	 * Configures the axis to show percent
	 */
	protected static void axisPercent(final IAxis axis)
	{
		final Range range = new Range(0.0, 100.0);
		final IRangePolicy rangePolicy = new RangePolicyFixedViewport();
		rangePolicy.setRange(range);
		axis.setRangePolicy(rangePolicy);
		axis.setMinorTickSpacing(50.0);
		axis.setMajorTickSpacing(50.0);
		axis.getAxisTitle().setTitle("%");
		axis.setFormatter(new LabelFormatterNumber(new DecimalFormat("###")));
	}
	
	/**
	 * Configures the axis to show nothing. This is intended for the x-axis of
	 * a bar chart. It only looks good when displayed small.
	 */
	protected static void axisBar(final IAxis axis)
	{
		final Range xRange = new Range(-0.1, 1.0);
		final IRangePolicy xRangePolicy = new RangePolicyFixedViewport();
		xRangePolicy.setRange(xRange);
		axis.setRangePolicy(xRangePolicy);
		axis.setPaintScale(false);
		axis.setFormatter(new LabelFormatterNull(1));
		axis.getAxisTitle().setTitle(null);
	}
	
	/**
	 * Helper function for adding a line trace to a chart, with a second fill
	 * trace underneath it in a slightly transparent colour.
	 * 
	 * @param chart chart to add the trace to
	 * @param newTraceSize the number of points in the Trace2DLtd that will be added
	 * @param newTraceTitle
	 * @param newTraceColor
	 * @param newFillAlpha
	 * @return the line trace that was created
	 */
	protected static ITrace2D addLineTrace(
			final Chart2D chart,
			final int newTraceSize,
			final String newTraceTitle,
			final Color newTraceColor,
			final int newFillAlpha)
	{
		final ITrace2D trace = new Trace2DLtd(newTraceSize);

		trace.setColor(newTraceColor);
		trace.setName(newTraceTitle);
		chart.addTrace(trace);
		final ITrace2D fillTrace = new Trace2DLtd(trace.getMaxSize());
		fillTrace.setColor(new Color(newTraceColor.getRed(), newTraceColor.getGreen(), newTraceColor.getBlue(), newFillAlpha));
		fillTrace.setName(null);
		fillTrace.setTracePainter(new TracePainterFill(chart));
		trace.addComputingTrace(fillTrace);
		chart.addTrace(fillTrace);
		fillTrace.setZIndex(trace.getZIndex() - 1);
		
		return trace;
	}
	
	/**
	 * Adds some points to the trace to ensure that it fills the whole x-axis
	 * before any data has been added.
	 */
	protected static void initLineTrace(final ITrace2D newTrace, final long updatePeriod)
	{
		for(int i = newTrace.getMaxSize(); i > 0; i--)
		{
			final ITracePoint2D point = new TracePoint2D(System.currentTimeMillis() - (updatePeriod * i), 0.0);
			newTrace.addPoint(point);
		}
	}
	
	/**
	 * Adds a trace that looks like a bar.
	 */
	protected static ITrace2D addBarTrace(
			final Chart2D chart,
			final String newTraceTitle,
			final Color newTraceColor)
	{
		final ITrace2D trace = new Trace2DLtd(2);
		
		trace.setColor(newTraceColor);
		trace.setName(newTraceTitle);
		chart.addTrace(trace);
		//TracePainterVerticalBar is awkward to get the right width, so do it this way instead.
		trace.setTracePainter(new TracePainterFill(chart));
		//Add a point at 0, otherwise the chart renders it wrong until it is updated.
		trace.addPoint(new TracePoint2D(0.0, 0.0));
		
		return trace;
	}
}
