package fpmon.stats;

import fpmon.chart.AxisTitlePainterPositioned;
import fpmon.chart.LabelFormatterBytes;
import fpmon.utils.StringUtils;
import info.monitorenter.gui.chart.*;
import info.monitorenter.gui.chart.IAxis.AxisTitle;
import info.monitorenter.gui.chart.rangepolicies.RangePolicyMinimumViewport;
import info.monitorenter.util.Range;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import org.hyperic.sigar.FileSystemUsage;
import org.hyperic.sigar.SigarException;
import org.hyperic.sigar.SigarProxy;

/**
 * Disc I/O rate and used space.
 *
 * @author Miles Asvachin
 */
public final class DiscStatCollector extends ChartStatCollector
{
	/**
	 * Y-axis will always show at least this high.
	 */
	private final static double yMin = 1024.0 * 100.0; //100KB

	/**
	 * Filesystem that is being displayed.
	 */
	private final String fileSystem;
	private final ITrace2D readTrace;
	private final ITrace2D writeTrace;
	/**
	 * Displays used space.
	 */
	private final ITrace2D barTrace;
	private final AxisTitle barTitle;
	
	private final Color readColor;
	private final Color writeColor;
	
	private long lastUpdateTime = 0;	
	private long lastUpdateReadKB = 0;
	private long lastUpdateWriteKB = 0;

	{
		title = "Disc";
		readColor = Color.YELLOW;
		writeColor = new Color(255, 128, 0);
		traceColor = new Color(255, 64, 0);
		traceTitleLength = 25;
	}
	
	/**
	 * @param container container in which this collector will display its data
	 * @param updatePeriod time in milliseconds between updates, used for initial x-axis calibration
	 * @param newFileSystem filesystem to display stats for
	 * @param sigar used here just to get the capacity of the filesystem to show in the title
	 */
	public DiscStatCollector(final JComponent container, final long updatePeriod, final String newFileSystem, final SigarProxy sigar)
	{
		container.setLayout(new GridBagLayout());
		container.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, rightBorder));
		container.setBackground(backgroundColor);
		
		fileSystem = newFileSystem;

		try
		{
			final FileSystemUsage fsUsage = sigar.getFileSystemUsage(fileSystem);
			title += String.format(" (%s, %s)", fileSystem.substring(0, 2), StringUtils.formatBytes(fsUsage.getTotal() * 1024));
		}
		catch(SigarException ex)
		{
			Logger.getLogger(DiscStatCollector.class.getName()).log(Level.WARNING, "Couldn't get filesystem info", ex);
		}
		
		final Chart2D lineChart = createLineChart(title, backgroundColor, foregroundColor);
		
		axisTime(lineChart.getAxisX(), timeAxisFormat);
		lineChart.getAxisY().setFormatter(new LabelFormatterBytes(true));
		lineChart.getAxisY().setRangePolicy(new RangePolicyMinimumViewport(new Range(0.0, yMin)));
		
		readTrace = addLineTrace(lineChart, traceSize, StringUtils.padRight("Read:", traceTitleLength), readColor, fillAlpha);
		initLineTrace(readTrace, updatePeriod);
		writeTrace = addLineTrace(lineChart, traceSize, "Write:", writeColor, fillAlpha);
		initLineTrace(writeTrace, updatePeriod);
		
		final GridBagConstraints lineChartConstraints = new GridBagConstraints();
		lineChartConstraints.fill = GridBagConstraints.BOTH;
		lineChartConstraints.weightx = 0.88;
		lineChartConstraints.weighty = 1.0;
		
		container.add(lineChart, lineChartConstraints);

		final Chart2D barChart = createBarChart(null, backgroundColor, foregroundColor);
		axisBar(barChart.getAxisX());
		axisPercent(barChart.getAxisY());
		barTitle = barChart.getAxisX().getAxisTitle();
		barTitle.setTitlePainter(new AxisTitlePainterPositioned(1.0, 0.0, 1.0, -2.4));
		barTitle.setTitleColor(traceColor);
		barTrace = addBarTrace(barChart, null, traceColor);
		
		final GridBagConstraints barChartConstraints = new GridBagConstraints();
		barChartConstraints.fill = GridBagConstraints.BOTH;
		barChartConstraints.weightx = 0.12;
		barChartConstraints.weighty = 1.0;
		
		container.add(barChart, barChartConstraints);
	}
	
	@Override
	protected void update(final SigarProxy sigar)
	{
		try
		{
			final FileSystemUsage fsUsage = sigar.getFileSystemUsage(fileSystem);
			final long currentTime = System.currentTimeMillis();
			final long deltaTime = currentTime - lastUpdateTime;
			
			//NOTE: both getDiskReadBytes and getDiskWriteBytes appear to be 
			//returning KB, not Bytes.
			final long readKB = fsUsage.getDiskReadBytes();
			
			//Hack! getDiskReadBytes and getDiskWriteBytes appear to be using 
			//unsigned ints somewhere underneath, so detect when they wrap, and 
			//adjust so the delta is always correct.
			if(readKB < lastUpdateReadKB)
			{
				//Multiply by 2 because Java Integers are signed.
				lastUpdateReadKB -= ((long)Integer.MAX_VALUE * 2L);
			}
			
			final long deltaReadBytes = (readKB - lastUpdateReadKB) * 1024L;
			final double readRate = (double)deltaReadBytes / (double)deltaTime;
			final ITracePoint2D readPoint = new TracePoint2D(currentTime, readRate);
			readTrace.addPoint(readPoint);
			readTrace.setName(StringUtils.padRight("Read: " + StringUtils.formatBytes((long)readRate) + "/s", traceTitleLength));
			
			final long writeKB = fsUsage.getDiskWriteBytes();
			
			if(writeKB < lastUpdateWriteKB)
			{
				lastUpdateWriteKB -= ((long)Integer.MAX_VALUE * 2L);
			}
			
			final long deltaWriteBytes = (writeKB - lastUpdateWriteKB) * 1024L;
			final double writeRate = (double)deltaWriteBytes / (double)deltaTime;
			final ITracePoint2D writePoint = new TracePoint2D(currentTime, writeRate);
			writeTrace.addPoint(writePoint);
			writeTrace.setName("Write: " + StringUtils.formatBytes((long)writeRate) + "/s");
			
			final double discUsePercent = fsUsage.getUsePercent() * 100.0;
			barTrace.addPoint(new TracePoint2D(0.0, discUsePercent));
			barTrace.addPoint(new TracePoint2D(1.0, discUsePercent));

			barTitle.setTitle(StringUtils.formatBytes(fsUsage.getUsed() * 1024));

			lastUpdateTime = currentTime;
			lastUpdateReadKB = readKB;
			lastUpdateWriteKB = writeKB;
		}
		catch(SigarException ex)
		{
			Logger.getLogger(CpuStatCollector.class.getName()).log(Level.WARNING, "Couldn't get filesystem info", ex);
		}
	}
}
