package fpmon.stats;

import fpmon.utils.IpAddressFinder;
import fpmon.utils.StringUtils;
import java.awt.*;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.JLabel;
import org.hyperic.sigar.SigarException;
import org.hyperic.sigar.SigarProxy;

/**
 * Collects miscellaneous stats.
 * 
 * @author Miles Asvachin
 */
public final class GeneralStatCollector extends StatCollector
{
	/**
	 * Format for the current date.
	 */
	private final DateFormat dateFormat = new SimpleDateFormat("EEE dd MMM yyyy");
	/**
	 * Format for the current time.
	 */
	private final DateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");

	private final JLabel hostNameLabel;
	private final JLabel localAddressLabel;
	private final JLabel publicAddressLabel;
	private final JLabel currentTimeLabel;
	private final JLabel currentDateLabel;
	private final JLabel uptimeLabel;
	
	private final IpAddressFinder finder;
	private final long addressUpdatePeriod;
	private long lastAddressUpdateTime = 0;

	public GeneralStatCollector(final JComponent container, final SigarProxy sigar, final Properties properties) throws MalformedURLException
	{
		container.setLayout(new GridBagLayout());
		container.setBackground(backgroundColor);
		container.setForeground(foregroundColor);
		
		final GridBagConstraints titlesLayout = new GridBagConstraints();
		titlesLayout.anchor = GridBagConstraints.LINE_END;
		titlesLayout.insets = new Insets(0, 5, 0, 5);
		titlesLayout.gridx = 0;
		titlesLayout.weightx = 1.0;
		titlesLayout.weighty = 1.0;
		
		final GridBagConstraints valuesLayout = new GridBagConstraints();
		valuesLayout.anchor = GridBagConstraints.LINE_START;
		valuesLayout.gridx = 1;
		valuesLayout.weightx = 1.0;
		valuesLayout.weighty = 1.0;

		hostNameLabel = addLabel("Name:", "localhost", foregroundColor, container, titlesLayout, valuesLayout);
		localAddressLabel = addLabel("Local:", "127.0.0.1", foregroundColor, container, titlesLayout, valuesLayout);
		publicAddressLabel = addLabel("Public:", "0.0.0.0", foregroundColor, container, titlesLayout, valuesLayout);
		
		currentTimeLabel = addLabel("Time:", "--:--:--", foregroundColor, container, titlesLayout, valuesLayout);
		currentDateLabel = addLabel("Date:", "---", foregroundColor, container, titlesLayout, valuesLayout);
		uptimeLabel = addLabel("Uptime:", "--:--:--", foregroundColor, container, titlesLayout, valuesLayout);
		
		//Note that the property is in seconds, but the field is in milliseconds, hence the factor of 1000.
		addressUpdatePeriod = Long.parseLong(properties.getProperty("AddressUpdatePeriod")) * 1000;
		finder = new IpAddressFinder();
	}
	
	/**
	 * Adds two labels as a key/value pair.
	 * 
	 * @param key title of the first label
	 * @param value title of the second label
	 * @param labelColor text colour
	 * @param labelContainer to put the labels in
	 * @param keyConstraints for positioning the key label
	 * @param valueConstraints for positioning the value label
	 * @return the value label
	 */
	private static JLabel addLabel(
			final String key,
			final String value,
			final Color labelColor,
			final JComponent labelContainer,
			final Object keyConstraints,
			final Object valueConstraints)
	{
		final JLabel keyLabel = new JLabel(key);
		keyLabel.setForeground(labelColor.darker().darker());
		keyLabel.setFont(keyLabel.getFont().deriveFont(0, 16));
		labelContainer.add(keyLabel, keyConstraints);
		
		final JLabel valueLabel = new JLabel(value);
		valueLabel.setForeground(labelColor);
		labelContainer.add(valueLabel, valueConstraints);
		
		try (final InputStream fontStream = ClassLoader.getSystemResourceAsStream("fpmon/gui/ocra10.ttf"))
		{
			final Font labelFont = Font.createFont(Font.TRUETYPE_FONT, fontStream).deriveFont(Font.PLAIN, 14);
			
			keyLabel.setFont(labelFont);
			valueLabel.setFont(labelFont);
		}
		catch(FontFormatException | IOException ex)
		{
			Logger.getLogger(GeneralStatCollector.class.getName()).log(Level.SEVERE, null, ex);
		}
		
		return valueLabel;
	}

	@Override
	protected void update(SigarProxy sigar)
	{
		updateAddresses();
		
		final Date currentDate = new Date(System.currentTimeMillis());
		currentTimeLabel.setText(timeFormat.format(currentDate));
		currentDateLabel.setText(dateFormat.format(currentDate));
		
		try
		{
			uptimeLabel.setText(StringUtils.formatTimeSpan((long)sigar.getUptime().getUptime()));
		}
		catch(SigarException ex)
		{
			Logger.getLogger(GeneralStatCollector.class.getName()).log(Level.WARNING, null, ex);
		}
	}
	
	/**
	 * Updates the host name and IP address labels.
	 * @todo this could block for a while, so it should have its own scheduler.
	 */
	private void updateAddresses()
	{
		final long currentTime = System.currentTimeMillis();
		
		if(currentTime - lastAddressUpdateTime > addressUpdatePeriod)
		{
			try
			{
				final InetAddress localHost = InetAddress.getLocalHost();
				final String hostName = localHost.getHostName();
				final String localAddress = localHost.getHostAddress();
				
				hostNameLabel.setText(hostName);
				hostNameLabel.setForeground(foregroundColor);
				localAddressLabel.setText(localAddress);
				localAddressLabel.setForeground(foregroundColor);

			}
			catch(UnknownHostException ex)
			{
				Logger.getLogger(GeneralStatCollector.class.getName()).log(Level.WARNING, "Could not resolve local host", ex);

				hostNameLabel.setForeground(Color.RED);
				localAddressLabel.setForeground(Color.RED);
			}

			final String publicAddress = finder.gePublicIpAddress();
			publicAddressLabel.setText(publicAddress == null ? "0.0.0.0" : publicAddress);
			
			switch(finder.publicAddressState())
			{
				case -1:
					publicAddressLabel.setForeground(Color.RED);
					break;
				case 0:
					publicAddressLabel.setForeground(foregroundColor);
					break;
				case 1:
					publicAddressLabel.setForeground(Color.RED);
					break;
				case 2:
					publicAddressLabel.setForeground(Color.GREEN);
					break;
				case 3:
					publicAddressLabel.setForeground(Color.ORANGE);
					break;
				default:
					publicAddressLabel.setForeground(Color.RED);
					break;
			}
			
			lastAddressUpdateTime = System.currentTimeMillis();
		}
	}
}
