package fpmon.stats;

import fpmon.chart.AxisTitlePainterPositioned;
import fpmon.chart.LabelFormatterBytes;
import fpmon.utils.StringUtils;
import info.monitorenter.gui.chart.*;
import info.monitorenter.gui.chart.rangepolicies.RangePolicyMinimumViewport;
import info.monitorenter.util.Range;
import java.awt.Color;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import org.hyperic.sigar.*;

/**
 *
 * @author Miles Asvachin
 */
public final class NetStatCollector extends ChartStatCollector
{
	/**
	 * Y-axis will always show at least this high.
	 */
	private final static double yMin = 1024.0 * 100.0; //100KB
	
	/**
	 * Interface that is being displayed.
	 */
	private final String netInterface;

	private final ITrace2D rxTrace;
	private final ITrace2D txTrace;
	
	private final Color txTraceColor;
	
	private long lastUpdateTime = 0;
	private long lastUpdateRx = 0;
	private long lastUpdateTx = 0;

	{
		title = "Network";
		traceColor = Color.BLUE;
		txTraceColor = Color.CYAN;
	}

	/**
	 * 
	 * @param container container in which this collector will display its data
	 * @param updatePeriod time in milliseconds between updates, used for initial x-axis calibration
	 * @param newNetInterface interface to display stats for
	 * @param sigar used here just to get the link speed to show in the title
	 * @throws SigarException if the interface could not be found
	 */
	public NetStatCollector(final JComponent container, final long updatePeriod, final String newNetInterface, final SigarProxy sigar) throws SigarException
	{
		container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
		container.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, rightBorder));
		container.setBackground(backgroundColor);

		netInterface = newNetInterface;
		final NetInterfaceStat netStat = sigar.getNetInterfaceStat(netInterface);
		final long speed = netStat.getSpeed();
		//title = netInterface + " (" + StringUtils.formatBits(speed) + "/s)";
		title += String.format(" (%s, %s/s)", netInterface, StringUtils.formatBits(speed));

		final Chart2D lineChart = createLineChart(title, backgroundColor, foregroundColor);
		axisTime(lineChart.getAxisX(), timeAxisFormat);
		lineChart.getAxisY().setFormatter(new LabelFormatterBytes(true));
		lineChart.getAxisY().setRangePolicy(new RangePolicyMinimumViewport(new Range(0.0, yMin)));
		
		rxTrace = addLineTrace(lineChart, traceSize, StringUtils.padRight("Rx:", traceTitleLength), traceColor, fillAlpha);
		initLineTrace(rxTrace, updatePeriod);
		txTrace = addLineTrace(lineChart, traceSize, StringUtils.padRight("Tx:", traceTitleLength), txTraceColor, fillAlpha);
		initLineTrace(txTrace, updatePeriod);

		container.add(lineChart);
	}
	
	@Override
	protected void update(final SigarProxy sigar)
	{
		try
		{
			final NetInterfaceStat netStat = sigar.getNetInterfaceStat(netInterface);
			final long currentTime = System.currentTimeMillis();
			final long deltaTime = currentTime - lastUpdateTime;
			
			final long speed = netStat.getSpeed();
			final long rx = netStat.getRxBytes();
			final long deltaRx = rx - lastUpdateRx;
			final long rxUse = Math.round((double)deltaRx / ((double)deltaTime / 1000.0));
			final ITracePoint2D rxPoint = new TracePoint2D(currentTime, rxUse);
			rxTrace.addPoint(rxPoint);
			
			final String rxUseString = StringUtils.formatBytes(rxUse) + "/s";
			final double rxPercent = ((double)rxUse / (double)speed) * 100.0;
			rxTrace.setName(StringUtils.padRight("Rx: " + rxUseString + " (" + Math.round(rxPercent) + "%)", traceTitleLength));
			
			final long tx = netStat.getTxBytes();
			final long deltaTx = tx - lastUpdateTx;
			final long txUse = Math.round((double)deltaTx / ((double)deltaTime / 1000.0));
			final ITracePoint2D txPoint = new TracePoint2D(currentTime, txUse);
			txTrace.addPoint(txPoint);
			
			final String txUseString = StringUtils.formatBytes(txUse) + "/s";
			final double txPercent = ((double)txUse / (double)speed) * 100.0;
			txTrace.setName(StringUtils.padRight("Tx: " + txUseString + " (" + Math.round(txPercent) + "%)", traceTitleLength));
			
			lastUpdateTime = currentTime;
			lastUpdateRx = rx;
			lastUpdateTx = tx;
		}
		catch(SigarException ex)
		{
			Logger.getLogger(CpuStatCollector.class.getName()).log(Level.WARNING, null, ex);
		}
	}
}
