package fpmon.utils;

/**
 * Various useful string-related functions.
 *
 * @author Miles Asvachin
 */
public final class StringUtils
{
	private static final String[] bitSizes = {"b", "Kb", "Mb", "Gb", "Tb", "Pb", "Eb", "Zb", "Yb"};
	private static final String[] byteSizes = {"B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"};

	private StringUtils()
	{
		throw new AssertionError();
	}
	
	public static String getBitSize(final int index)
	{
		return bitSizes[index];
	}
	
	public static String getByteSize(final int index)
	{
		return byteSizes[index];
	}
	
	/**
	 * Formats a number of bits into the largest decimal unit that it will be at
	 * least 1 in. e.g. 999 => 999b, whereas 1000 => 1Kb.
	 */
	public static String formatBits(final long bits)
	{
		if(bits < 1000L)
		{
			return bits + bitSizes[0];
		}

		final int exp = getBitSize(bits);
		
		return String.format("%.1f%s", bits / Math.pow(1000L, exp), bitSizes[exp]);
	}
	
	public static int getBitSize(final long bits)
	{
		return bits < 1000L ? 0 : (int)(Math.log(bits) / Math.log(1000L));
	}
	
	/**
	 * Formats a number of bytes into the largest binary unit that it will be at
	 * least 1 in. e.g. 1023 => 1023B, whereas 1024 => 1KB.
	 */
	public static String formatBytes(final long bytes)
	{
		if(bytes < 1024L)
		{
			return bytes + byteSizes[0];
		}

		final int exp = getByteSize(bytes);
		
		return String.format("%.1f%s", bytes / Math.pow(1024L, exp), byteSizes[exp]);
	}
	
	public static int getByteSize(final long bytes)
	{
		return bytes < 1024L ? 0 : (int)(Math.log(bytes) / Math.log(1024L));
	}
	
	/**
	 * Very basic formatting of a duration in seconds into days+hours+minutes+seconds
	 */
	public static String formatTimeSpan(final long seconds)
	{
		final StringBuilder result = new StringBuilder();

		long remainder = seconds;
		final int s = (int)(remainder % 60);
		remainder -= s;
		final int m = (int)((remainder / 60) % 60);
		remainder -= 60 * m;
		final int h = (int)((remainder / (60 * 60)) % 24);
		remainder -= 60 * 60 * h;
		final int d = (int)(remainder / (60 * 60 * 24));
		
		if(d > 0)
		{
			result.append(d).append("d ");
		}
		
		if(h > 0)
		{
			result.append(h).append("h ");
		}
		
		if(m > 0)
		{
			result.append(m).append("m ");
		}
		
		result.append(s).append("s");
		
		return result.toString();
	}
	
	/**
	 * Pads a string with spaces on the right.
	 */
	public static String padRight(String s, int n)
	{
		return String.format("%1$-" + n + "s", s);
	}

	/**
	 * Pads a string with spaces on the left.
	 */
	public static String padLeft(String s, int n)
	{
		return String.format("%1$#" + n + "s", s);
	}
}
